@@ -23,6 +23,7 @@ pub struct Artifacts {
 }
 
 impl Build {
+    #[allow(clippy::new_without_default)]
     pub fn new() -> Build {
         Build {
             out_dir: env::var_os("OUT_DIR").map(|s| PathBuf::from(s).join("lua-build")),
@@ -76,7 +77,8 @@ impl Build {
             .target(target)
             .host(host)
             .warnings(false)
-            .opt_level(2);
+            .opt_level(2)
+            .cargo_metadata(false);
 
         match target {
             _ if target.contains("linux") => {
@@ -86,10 +88,13 @@ impl Build {
                 config.define("LUA_USE_LINUX", None);
             }
             _ if target.contains("apple-darwin") => {
-                config.define("LUA_USE_MACOSX", None);
+                match version {
+                    Lua51 => config.define("LUA_USE_LINUX", None),
+                    _ => config.define("LUA_USE_MACOSX", None),
+                };
             }
-            _ if target.contains("emscripten") => {}
             _ if target.contains("windows") => {
+                // Defined in Lua >= 5.3
                 config.define("LUA_USE_WINDOWS", None);
             }
             _ => panic!("don't know how to build Lua for {}", target),
@@ -108,7 +113,8 @@ impl Build {
 
         config
             .include(&source_dir)
-            .flag("-w")
+            .flag("-w") // Suppress all warnings
+            .flag_if_supported("-fno-common") // Compile common globals like normal definitions
             .file(source_dir.join("lapi.c"))
             .file(source_dir.join("lauxlib.c"))
             .file(source_dir.join("lbaselib.c"))
@@ -173,8 +179,8 @@ impl Build {
         }
 
         Artifacts {
-            lib_dir: lib_dir,
-            include_dir: include_dir,
+            lib_dir,
+            include_dir,
             libs: vec![lib_name.to_string()],
         }
     }
